def solve(E: str, D: int, M: int, Y: int) -> str:
    """
    E: Name of event
    D: Day of event
    M: Month of event
    Y: Year of event
    """
    gtaD, gtaM, gtaY = 19, 11, 2026
    if (Y, M, D) < (gtaY, gtaM, gtaD):
        return f"we got {E} before gta6"
    else:
        return f"we got gta6 before {E}"

def main():
    T = int(input())
    for _ in range(T):
        E = input()
        temp = input().split()
        Y, M, D = int(temp[0]), int(temp[1]), int(temp[2])
        print(solve(E, D, M, Y))

if __name__ == '__main__':
    main()


import requests
import json
# from .problem import Problem
from datetime import datetime, timedelta
from zoneinfo import ZoneInfo

BASE_URL = 'https://calicojudge.com/api/v4'

USER = None

CONTEST_ID = '-1'

def _request(method: str,
             endpoint: str,
             data=None,
             files=None,
             require_200=True):
    print("Request: " + method + " " + endpoint)
    r = requests.request(method, BASE_URL + endpoint,
                      data=data,
                      files=files,
                      auth=USER)
    print(f'STATUS: {r.status_code}')
    print(f'{r.text[:200]}')
    if require_200 and r.status_code >= 300:
        raise Exception(r.status_code)
    print(json.dumps(r.json(), indent=2))
    return r

def set_user(user_password_pair: tuple[str, str]):
    """
    Set the user used for api requests
    """
    global USER
    USER = user_password_pair

def set_contest_id(contest_id: str):
    global CONTEST_ID
    CONTEST_ID = contest_id

def upload_problem_zip(file_name, pid: str|None) -> str:
    data = None
    if pid is None:
        print('Creating problem...')
    else:
        print(f'Replacing problem; pid: {pid}...')
        data = {'problem': str(pid), 'color': '#ffffff'}
    r = _request('post',
                 f'/contests/{CONTEST_ID}/problems',
                 data=data,
                 files={'zip': open(file_name, 'rb')})

    print(f"problem uploaded with pid: {pid}")
    pid = r.json()['problem_id']
    assert pid is not None
    return pid

def unlink_problem_from_contest(pid: str):
    _ = _request('DELETE', f'/contests/{CONTEST_ID}/problems/{pid}')
    print('Unlinking problem...')

def link_problem_to_contest(pid: str, label: str, rgb: str):
    # TODO: support points
    """
    {
      "label": "string",
      "color": "string",
      "rgb": "string",
      "points": 1,
      "lazy_eval_results": 0
    }
    """
    data = {
            'label': label,
            'rgb': rgb,
            }
    # data = json.dumps(data)
    print(f'Linking problem... {pid}')
    _ = _request(
            'PUT',
            f'/contests/{CONTEST_ID}/problems/{pid}',
            data = data)
            # files={'data': ('problems.json', data)})
    return

def get_problem(pid: str):
    r = _request(
            'get',
            f'/contests/{CONTEST_ID}/problems/{pid}',
            require_200=False)
    if r.status_code == 404:
        return None
    if r.status_code >= 300:
        raise Exception(r.status_code)
    return r.json()


def add_problem_metadata_to_contest(name: str, label: str, rgb: str):
    """Adds the problem metadata, but not the zip"""
    # try:
    #     _request('delete', f'/contests/{CONTEST_ID}/problems/{pid}')
    # except Exception as e:
    #     print("delete failed: " + str(e))
    data = [{
            'id': name,
            'label': label,
            'rgb': rgb,
            }]
    data = json.dumps(data)
    print(f'Adding problem metadata {data}')
    r = _request(
            'post',
            f'/contests/{CONTEST_ID}/problems/add-data',
            # data = {'problem': str(pid)},
            files={'data': ('problems.json', data)})
    r = r.json()
    assert len(r) == 1
    return r[0]

def create_contest(cid: str, name: str, start_time: datetime = datetime(2000, 1, 1, 0, 0), duration: str = '9999999:00:00'):
    """
    Creates a contest, check code for parameters used. Assumes datetime object is in
    Pacific time.
    """
    def to_iso8601_pacific(dt: datetime) -> str:
        """Assume input datetime is in Pacific Time and return ISO 8601 string."""
        pacific = ZoneInfo("America/Los_Angeles")
        dt = dt.replace(tzinfo=pacific)
        return dt.isoformat()

    activate_time = start_time - timedelta(minutes=50)
    data = {
            'id': cid,
            'name': name,
            'start_time': to_iso8601_pacific(start_time),
            'duration': duration,
            'penalty_time': 10,
            'activate_time': to_iso8601_pacific(activate_time),
            'scoreboard_freeze_time': '+02:00:00',
            'scoreboard_freeze_duration': '1:00:00',
            }

    data = json.dumps(data)
    print(f'Creating contest {data}')
    _ = _request(
            'post',
            '/contests',
            files={'json': ('contest.json', data)})

# r = requests.get(BASE_URL + '/status', auth=USER)




def solve(E: str, D: int, M: int, Y: int) -> str:
    """

    E: The name of the event
    D: Day
    M: Month
    Y: Year 
    """
    # YOUR CODE HERE
    return ""

def main():
    T = int(input())
    for _ in range(T):
        E = input()
        temp = input().split()
        D, M, Y = int(temp[0]), int(temp[1]), int(temp[2])
        print(solve(E, D, M, Y))

if __name__ == 'main':
    main()


from .problem import Problem, TestFileBase
from abc import ABC, abstractmethod
from collections.abc import Collection, Iterable
# from typing import override

class TestCaseBase(ABC):
    def __init__(self) -> None:
        pass

    @abstractmethod
    def write_test_in(self):
        """Write the input file of this test case using print_test"""
        pass

    @abstractmethod
    def verify_case(self, test_sets):
        pass

class MulticaseTestFile(TestFileBase):
    problem = None

    def __init__(self, cases: Iterable[TestCaseBase]|None = None) -> None:
        if cases is None:
            self.cases: list[TestCaseBase] = []
        else:
            self.cases = list(cases)
        super().__init__()

    # @override
    # def get_subproblems(self) -> list[str]:
    #     return ['bonus']

    # @override
    def write_test_in(self):
        assert self.problem != None, "Must set problem for multicase test file"
        self.problem.print_test(len(self.cases))
        for case in self.cases:
            case.write_test_in()
        return super().write_test_in()

#
# class TestFileFromFile(TestFileBase):
#
#     @override
#     def write_test_in(self):
#         return super().write_test_in()



import os
import sys
import zipfile

# Functions from the old calico lib.
def get_path(is_secret, file_name='', ext=None):
    """
    If file_name is unspecified, makes a path to the root of the sample or
    secret directory depending on is_secret.

    Otherwise, get the path for a file in the sample or secret directory. The
    path is absolute based on the location of this file, not relative to the
    execution path.
    """
    relative_path = ''.join([
        'data/',
        'secret/' if is_secret else 'sample/',
        ('secret_' if is_secret else 'sample_') if file_name else '',
        file_name,
        f'.{ext}' if ext else ''
    ])
    return os.path.join(os.path.dirname(__file__), relative_path)

def delete_old_zips(problem_name, test_set_names):
    """
    Delete old zips if they exist so we can start making with a clean slate.
    """
    print('Deleting old zips', end='...')
    deleted = False
    for test_set_name in test_set_names:
        zip_file_name = get_zip_file_path(problem_name, test_set_name)
        if os.path.exists(zip_file_name):
            os.remove(zip_file_name)
            deleted = True
    print('Done!' if deleted else 'Nothing to delete!')


def make_actual_zips(problem_name, time_limit, test_set_names, \
                     is_data_in_test_set, is_submission_in_test_set):
    """
    Create a zip for each test set. Each test set consists of data, submissions,
    and the DOMjudge metadata file.
    """
    for test_set_name in test_set_names:
        print(f'Creating zip for test set "{test_set_name}"...')

        file_path = get_zip_file_path(problem_name, test_set_name)
        with zipfile.ZipFile(file_path, 'w', zipfile.ZIP_DEFLATED) as zip_file:
            zip_path(zip_file, 'data', test_set_name, is_data_in_test_set)
            zip_path(zip_file, 'submissions', test_set_name, is_submission_in_test_set)
            zip_metadata(zip_file, problem_name, test_set_name, time_limit)

        print(f'Done creating zip for test set "{test_set_name}"!')


def get_zip_file_path(problem_name, test_set_name):
    """
    Get the path for a zip file given the name of the test set and the DOMjudge
    metadata name.
    """
    return f'{problem_name}_{test_set_name}.zip'


def zip_path(zip_file, path, test_set_name, is_file_in_test_set):
    """
    Add all files in path. Only files that is_file_in_test_set says
    are to be added to the current test_set_name will be added.
    """
    print(f'Zipping directory "{path}" for test set "{test_set_name}"', end='...')

    # path = os.path.join(os.path.dirname(__file__), relative_path)
    for root, dirs, files in os.walk(path):
        for file in files:
            if is_file_in_test_set(file, test_set_name):
                file_path = os.path.join(root, file)
                zip_path = os.path.relpath(os.path.join(root, file), os.path.join(path, '..'))
                zip_file.write(file_path, zip_path)

    print('Done!')


def zip_metadata(zip_file,
                 problem_name,
                 test_set_name,
                 time_limit,
                 custom_compare = None):
    """
    Add the DOMjudge metadata file to the zip_file with the test_set_name. This
    function creates a temporary file, writes name and timelimit, adds it to
    the zip, then deletes the temporary file.
    """
    print(f'Zipping domjudge-problem.ini for test set "{test_set_name}"', end='...')

    meta_path = os.path.join(os.path.dirname(__file__), 'domjudge-problem.ini')
    with open(meta_path, 'w', encoding='utf-8', newline='\n') as meta_file:
        problem_name = problem_name
        print(f'name={problem_name}_{test_set_name}', file=meta_file)
        print(f'timelimit={time_limit}', file=meta_file)
        if custom_compare is not None:
            print(f'special_compare=\'{custom_compare}\'', file=meta_file)

    zip_file.write(meta_path, 'domjudge-problem.ini')
    os.remove(meta_path)

    print('Done!')



import argparse
from dataclasses import dataclass, field
from datetime import datetime
from typing import List, Literal

from .judge_api import create_contest, set_contest_id
from .problem import Problem
from calico_lib import judge_api

@dataclass
class Contest():
    contest_id: str
    name: str
    start_time: datetime = datetime(2000, 1, 1, 0, 0)
    duration: str = '9999999:00:00'
    problems: List[Problem] = field(default_factory=list)

    def create_contest(self):
        create_contest(self.contest_id, self.name, self.start_time, self.duration)
        print('=======================')
        print('TODO: make the contest private, not available for all teams, and add the appropriate groups.')
        print('=======================')

def link_external_problem(cid, pid, label, rank):
    rank_color_map = {
            1: '#e9e4d7',
            2: '#ff7e34',
            3: '#995d59',
            4: '#000000',
            }
    set_contest_id(cid)
    judge_api.link_problem_to_contest(pid, label, rank_color_map[rank])



def solve(A: int, B: int) -> int:
    """
    Return the sum of A and B.
    
    A: a non-negative integer
    B: another non-negative integer
    """
    assert 1 <= A <= 1e12
    assert 1 <= B <= 1e12
    return 0


def main():
    T = int(input())
    assert T <= 100
    for _ in range(T):
        temp = input().split()
        A, B = int(temp[0]), int(temp[1])

if __name__ == '__main__':
    main()




import tomllib

from calico_lib import judge_api

def try_load_toml(file_path):
    try:
        with open(file_path, 'rb') as f:
            toml = tomllib.load(f)
            return toml
    except Exception as e:
        print('Warning: unable to load some configs...' + str(e))
        return None


def load_secrets(file_path: str = 'secrets.toml'):
    secrets = try_load_toml(file_path)
    if secrets is None:
        return
    judge_api.set_user(
            (secrets['username'], secrets['password']))

def load_configs(file_path: str = 'config.toml'):
    config = try_load_toml(file_path)
    if config is None:
        return
    with open(file_path, 'rb') as f:
        config = tomllib.load(f)
    judge_api.set_contest_id(config['default_contest_id'])



import os
import shutil
import sys

from calico_lib.config import load_secrets, load_configs
from .contest import Contest
from .judge_api import set_contest_id, set_user
from .problem import Problem
import argparse


def run_cli(obj: Contest|Problem):
    desc = 'CLI interface for various actions for this problem. '\
            'By default, generates and verifies test cases.' \
            'Specify contest ID and password specified in ../config.toml, and password in ../secrets.toml, ' \
            'or use command line arguments, which takes priority.'
    if isinstance(obj, Contest):
        desc = 'CLI interface for various actions for this contest. '\
                'Similar to problem CLI, but operates on all problems, or a specific one.'
    parser = argparse.ArgumentParser(
                    prog='CALICOLib CLI',
                    description=desc,
                    epilog='')

    parser.add_argument('-a', '--auth', help='Username and password for judge, separated by colon.')
    parser.add_argument('-c', '--cid', type=str, help='Specify an alternative contest id.')
    parser.add_argument('-u', '--upload', action='store_true', help='Create or update the problem on the judge. Defaults to a draft version, unless -f is specified.')
    parser.add_argument('-l', '--link', action='store_true', help='Link the problem to the contest.')
    # parser.add_argument('-L', '--unlink', action='store_true', help='Unlink the problem from the contest.')
    parser.add_argument('-s', '--skip-test-gen', action='store_true', help='Skip test generation.')
    parser.add_argument('-f', '--final', action='store_true', help='Don\'t append _draft to the problem id.')
    # parser.add_argument('-i', '--p-ord', type=int, help='Problem order.')

    if isinstance(obj, Contest):
        parser.add_argument(
                '-n', '--create', action='store_true',
                help="Create the contest."
                )
        parser.add_argument(
                '-z', '--contest-zip', action='store_true',
                help="Create the contest folder, used for creating contest.zip."
                )
        parser.add_argument(
                '-p', '--target-problem', type=str,
                help="Operate on a specific problem."
                )

    if isinstance(obj, Contest):
        load_secrets()
        set_contest_id(obj.contest_id)
    elif isinstance(obj, Problem):
        load_secrets('../secrets.toml')
        load_configs('../config.toml')

    args = parser.parse_args()
    # if isinstance(obj, Contest) and len(sys.argv) == 1:
    #     parser.print_help()
    #     return

    if isinstance(obj, Contest) and args.create:
        obj.create_contest()
        return

    target_problems = None
    if isinstance(obj, Problem):
        target_problems = [obj]
    elif args.target_problem is not None:
        assert isinstance(obj, Contest)
        for i in obj.problems:
            if i.problem_name == args.target_problem:
                target_problems = [i]
                break

    else:
        target_problems = obj.problems

    if args.auth is not None:
        set_user(tuple(args.auth.split(':')))
    if args.cid is not None:
        set_contest_id(args.cid)

    assert target_problems is not None

    zip_dir_name = 'contest_zip_stuff'
    if args.contest_zip:
        try:
            shutil.rmtree(zip_dir_name)
        except FileNotFoundError:
            # First time running
            pass
        os.makedirs(zip_dir_name)

    for target_problem in target_problems:
        print(f'======\n---> Operating on {target_problem.problem_name}\n======')
        os.chdir(target_problem.problem_dir)
        target_problem.init_problem()

        if args.final:
            target_problem.problem_name = target_problem.problem_name
            # if args.p_ord is not None:
            #     target_problem.ordinal = args.p_ord
            assert target_problem.ordinal != -1
        else:
            target_problem.problem_name = target_problem.problem_name + '_draft'

        if not args.skip_test_gen:
            if not target_problem.always_skip_test_gen:
                print('\n=== Creating Tests ===')
                target_problem.create_all_tests()

            print('\n=== Creating Zip ===')
            target_problem.create_zip('')

        def ignore(path, names):
            # print(path, names)
            inc = ['./data/sample', './templates']
            if path == '.':
                return [name for name in names if name not in ['data', 'templates']]
            if path == './data':
                return [name for name in names if name not in ['sample']]
            if path in inc:
                return []
            return names
        if args.contest_zip:
            shutil.copytree('.', f'../{zip_dir_name}/{target_problem.problem_name}', ignore=ignore)

        if args.upload:
            print('=== Uploading Problem Zip ===')
            target_problem.upload()

        # if args.unlink:
        #     assert not args.link
        #     print('=== Unlinking from Contest ===')
        #     target_problem.link_to_contest()
        if args.link:
            print('=== Linking to Contest ===')
            target_problem.link_to_contest()

Problem._cli_func = run_cli



#!/usr/bin/env python3

from collections.abc import Collection, Iterable
from typing import override
from calico_lib import Problem, py_runner, TestFileBase
import random

class TestCase():
    def __init__(self, X: int, Y: int) -> None:
        self.X = X
        self.Y = Y
        super().__init__()

    def write_test_in(self):
        """Write the input file of this test case using print_test"""
        p.print_test(self.X, self.Y)

    def verify_case(self, test_sets):
        assert 1 <= self.X <= 10000
        if 'main' in test_sets:
            assert self.X <= 100

solution = py_runner('submissions/accepted/add_sol.py')

# TODO: move this to library
class Test(TestFileBase):

    def __init__(self, cases: Iterable[TestCase]|None = None) -> None:
        if cases is None:
            self.cases: list[TestCase] = []
        else:
            self.cases = list(cases)
        super().__init__()

    # @override
    # def get_subproblems(self) -> list[str]:
    #     return ['bonus']

    @override
    def write_test_in(self):
        p.print_test(len(self.cases))
        for case in self.cases:
            case.write_test_in()
        return super().write_test_in()

    @override
    def write_test_out(self, infile: str):
        p.print_test(solution.exec_file(infile))

    @override
    def validate_test_in(self, infile: str):
        """Verify the test using assert (not recommended, consider properly
        verifying by reading the file)."""
        total = 0
        assert 1 <= len(self.cases) <= 100, f"Got {len(self.cases)} cases"
        for case in self.cases:
            case.verify_case(self.subproblems)
            total += case.X + case.Y
        assert total <= 1e6

p = Problem[Test](
        'add',
        test_sets=['main', 'bonus'])

p.add_sample_test(Test([
    TestCase(4, 7),
    TestCase(1, 23),
    TestCase(9, 8),
    TestCase(1, 1),
    ]))

@p.hidden_test_generator(test_count=5, subproblems=['main', 'bonus'])
def pure_random() -> Test:
    test = Test()
    for i in range(100):
        test.cases.append(TestCase(random.randint(1, 100), random.randint(1, 100)))
    return test

@p.hidden_test_generator(test_count=5, subproblems=['bonus'])
def pure_random2():
    cases = (TestCase(random.randint(70, 10000), random.randint(70, 10000)) for _ in range(5))
    return Test(cases)

def main():
    # p.run_cli()
    p.create_all_tests()
    # p.create_zip()

main()



"""CALICO lib for all your problem writing needs"""

__version__ = "0.1.19"

from .problem import Problem, TestFileBase, Subproblem
from .contest import Contest
from .runner import *
from .cli import run_cli

from .multicase import TestCaseBase, MulticaseTestFile



from abc import ABC, abstractmethod
from collections.abc import Callable, Collection
import os
import shutil
from typing import Dict, NamedTuple
# from warnings import deprecated
import zipfile

from .judge_api import add_problem_metadata_to_contest, get_problem, link_problem_to_contest, set_contest_id, set_user, unlink_problem_from_contest, upload_problem_zip
import argparse
from .legacy import *
import traceback
import subprocess

# TODO:
# screw problem dir, just have user cd into dir

class TestFileBase(ABC):
    # TODO: consider storing filename in this class

    subproblems: Collection[str]

    def __init__(self) -> None:
        # The list of subproblems this test should belong to
        self.subproblems = []

    @abstractmethod
    def write_test_in(self):
        """Write the input file of this test using print_test"""
        pass

    @abstractmethod
    def write_test_out(self, infile: str):
        """Write the solution file, with input already written in infile"""
        pass

    @abstractmethod
    def validate_test_in(self, infile: str):
        """Validate the current test in, written in infile"""
        # assert False, "Must validate test"
        pass

# A test consist of either a single case or multiple test cases

_DEFAULT_MEMLIMIT = 256_000_000

class Subproblem(NamedTuple):
    name: str
    rank: int
    time_limit: int = 1
    mem_limit: int = _DEFAULT_MEMLIMIT
    def color(self):
        rank_color_map = {
                1: '#e9e4d7',
                2: '#ff7e34',
                3: '#995d59',
                4: '#000000',
                }
        return rank_color_map[self.rank]

class Problem:
    test_sets: list[Subproblem]
    problem_dir: str
    custom_checker: None|str

    _cli_func: Callable|None = None

    def __init__(self, problem_name: str, problem_dir: str, test_sets: list[Subproblem] = []):
        self.problem_name = problem_name
        self.test_sets = test_sets
        self.problem_dir = problem_dir
        self.custom_checker = None

        # order of the problem in the contest. Used for label. Otherwise, label is problem_name
        self.ordinal = -1

        self.sample_count = 0
        self.hidden_count = 0

        self.always_skip_test_gen = False
        self.pre_fn = None

        # mapping from test sets to tests included in that test set
        self.test_paths: Dict[str, list[str]] = dict()
        for subproblem in test_sets:
            self.test_paths[subproblem.name] = []

        # the current file that we will write to with print_test
        self._cur_file = None
        self._sample_path = os.path.join('data', 'sample')
        self._secret_path = os.path.join('data', 'secret')
        self._all_test_generators = []


    def init_problem(self):
        """
        Create subdirectories for this problem
        """
        os.makedirs(os.path.join(self.problem_dir, 'submissions', 'accepted'), exist_ok=True)
        os.makedirs(os.path.join(self.problem_dir, 'submissions', 'run_time_error'), exist_ok=True)
        os.makedirs(os.path.join(self.problem_dir, 'submissions', 'time_limit_exceeded'), exist_ok=True)
        os.makedirs(os.path.join(self.problem_dir, 'submissions', 'wrong_answer'), exist_ok=True)
        os.makedirs(os.path.join(self.problem_dir, 'templates'), exist_ok=True)
        os.makedirs(os.path.join(self.problem_dir, 'scripts'), exist_ok=True)

    def add_test_set(self, problem_name: str, rank: int, time_limit = 1, mem_limit: int = _DEFAULT_MEMLIMIT):
        self.test_sets.append(Subproblem(problem_name, rank, time_limit, mem_limit))

    def print_test(
            self,
            *values: object,
            sep: str | None = " ",
            end: str | None = "\n",
            ):
        """Print data to the test file. Arguments are the same as print."""
        assert self._cur_file != None, "This function should be called in one of the test_write_* function"
        print(*values, sep=sep, end=end, file=self._cur_file)

    def _add_test(self,
                  test_file_or_fn: TestFileBase|Callable[[], TestFileBase],
                  file_dir: str,
                  file_prefix: str,
                  subproblems: list[str]|None = None):
        if subproblems is None:
            subproblems = [s.name for s in self.test_sets]
        file_path = os.path.join(file_dir, file_prefix + '_' + subproblems[0])
        def test_generator():
            if callable(test_file_or_fn):
                test = test_file_or_fn()
            else:
                test = test_file_or_fn
            test.subproblems = subproblems
            with open(file_path + '.in', 'w', encoding='utf-8', newline='\n') as in_file:
                self._cur_file = in_file
                print(f"Writing infile {file_path+'.in'}")
                test.write_test_in()
            self._cur_file = None

            # try:
            test.validate_test_in(file_path + '.in')
            # except (AssertionError, subprocess.CalledProcessError):
            #     print(f"!!--------------------------------------------")
            #     print(f"Validation failed on testcase {file_name}")
            #     print(traceback.format_exc())
            #     # pass
            with open(file_path + '.ans', 'w', encoding='utf-8', newline='\n') as out_file:
                self._cur_file = out_file
                print(f"Writing ans (out) file {file_path+'.ans'}")
                test.write_test_out(file_path + '.in')
            self._cur_file = None

        self._all_test_generators.append(test_generator)
        for subproblem in subproblems:
            self.test_paths[subproblem].append(file_path)

    def add_raw_test_NO_VALIDATE(self, path, subproblems: list[str]|None = None):
        if subproblems is None:
            subproblems = [s.name for s in self.test_sets]
        for subproblem in subproblems:
            self.test_paths[subproblem].append(path)

    def add_sample_test(self, test: TestFileBase, name: str='', subproblems: list[str]|None = None):
        if name != '': name = '_' + name
        self._add_test(test, self._sample_path, f'{self.sample_count:02d}{name}', subproblems)
        self.sample_count += 1

    def add_hidden_test(self, test_or_fn: TestFileBase|Callable[[], TestFileBase], name: str='', subproblems: list[str]|None = None):
        if isinstance(test_or_fn, TestFileBase):
            print(f'[Warning]: {self.problem_name} hidden test added in place...')
        if name != '': name = '_' + name
        self._add_test(test_or_fn, self._secret_path, f'{self.hidden_count:02d}{name}', subproblems)
        self.hidden_count += 1

    def hidden_test_generator(self, test_count = 1, subproblems: list[str] = ['main']):
        """A function decorator that adds a hidden test generator. Repeats to generate
        test_count number of test files.
        """
        def generator(gen_fn: Callable[[], TestFileBase]):
            for _ in range(test_count):
                self.add_hidden_test(gen_fn, gen_fn.__name__, subproblems)
            return gen_fn
        return generator

    def pre_gen_fn(self, fn: Callable[[], None]):
        self.pre_fn = fn
        return fn

    def test_validator(self, validator: Callable[[Collection[TestFileBase]], None]):
        self._test_validator = validator
        return validator

    def create_all_tests(self):
        """Delete existing tests and regenerate them based on all the tests and generators added."""
        os.chdir(self.problem_dir)

        try:
            shutil.rmtree(self._sample_path)
            shutil.rmtree(self._secret_path)
        except FileNotFoundError:
            # First time running
            pass
        os.makedirs(self._sample_path, exist_ok=True)
        os.makedirs(self._secret_path, exist_ok=True)

        if self.pre_fn is not None:
            print('\nRunning pre generation tasks...')
            self.pre_fn()
        for fn in self._all_test_generators:
            fn()

    def create_zip(self, name_prefix='draft_'):
        """
        Create a zip for each test set. Each test set consists of data, submissions,
        and the DOMjudge metadata file.
        """
        os.chdir(self.problem_dir)

        final_name = name_prefix + self.problem_name

        for test_set in self.test_sets:
            file_path = get_zip_file_path(final_name, test_set.name)
            file_path = os.path.join(self.problem_dir, file_path)
            print(f'Creating zip for test set "{test_set.name}" at "{file_path}...')
            with zipfile.ZipFile(file_path, 'w', zipfile.ZIP_DEFLATED) as zip_file:
                for file in self.test_paths[test_set.name]:
                    zip_file.write(file+'.in')
                    zip_file.write(file+'.ans')

                zip_path(zip_file, 'submissions', test_set.name, lambda _, _2: True)
                zip_metadata(zip_file,
                             final_name,
                             test_set.name,
                             test_set.time_limit,
                             self.custom_checker)

            print(f'Done creating zip for test set "{test_set.name}"!')

    def add_final_metadata(self, p_num: int):
        """
        DEPRECATED:
        Upload metadata to contest.
        """
        print("adding metadata")
        i = 0
        for sub_test in self.test_sets:
            subproblem = sub_test.name

            label = str(p_num)
            if i > 0:
                label = label + f'b{i}'
            add_problem_metadata_to_contest(
                    self.problem_name + '_' + subproblem,
                    label,
                    sub_test.color(),
                    )

    def upload(self):
        for test_set in self.test_sets:
            pid = self.problem_name + '_' + test_set.name
            label = pid
            judge_problem = get_problem(pid)
            i = 0
            if judge_problem is None:
                print('problem not found... creating problem')
                if self.ordinal != -1:
                    label = str(self.ordinal)
                    if i > 0:
                        label = label + f'b{i}'
                add_problem_metadata_to_contest(pid, label, test_set.color())
            pid = upload_problem_zip(get_zip_file_path(self.problem_name, test_set.name), pid)
            i = i + 1

    def link_to_contest(self):
        """
        Link to contest, ordinal is used for tag (1 if this is the first problem, -1 to use pid as tag).
        """
        i = 0
        for test_set in self.test_sets:
            pid = self.problem_name + '_' + test_set.name
            label = pid
            print("== Linking to Contest ==")
            # try:
            #     unlink_problem_from_contest(pid)
            # except Exception:
            #     pass
            if self.ordinal != -1:
                label = str(self.ordinal)
                if i > 0:
                    label = label + f'b{i}'
            judge_problem = get_problem(pid)
            if judge_problem is None:
                link_problem_to_contest(pid, label, test_set.color())
            else:
                print('Warning: problem already linked, skipping...')
            i = i + 1


    # @deprecated("Use 'from calico_lib import run_cli' instead.")
    def run_cli(self, pre_fn: Callable[[], None]|None = None):
        """
        DEPRECATED
        """
        """
        Run pre_fn before generating test cases.
        """
        if pre_fn is not None:
            assert self.pre_fn is None
            self.pre_fn = pre_fn
        assert self._cli_func is not None
        self._cli_func()



from collections.abc import Collection, Sequence
from dataclasses import dataclass
import os
import shutil
import subprocess
import sys


CC: str = 'g++'

_ALL_EXECUTABLES: list['Runner'] = []

def configure_cpp_cc(cmd):
    global CC
    CC = cmd

# Runs various source files
@dataclass
class Runner:
    run_cmd: Sequence[str]
    compile_cmd: Sequence[str] | None = None

    def __init__(self, run_cmd, compile_cmd):
        self.run_cmd = run_cmd
        self.compile_cmd = compile_cmd
        _ALL_EXECUTABLES.append(self)

    def exec(self):
        try:
            out = subprocess.check_output(self.run_cmd).decode()
        except subprocess.CalledProcessError as e:
            print('Runner failed to run:')
            print(self)
            print(e)
            raise
        return out

    def exec_file(self, infile: str):
        with open(infile, encoding='utf-8', newline='\n') as file:
            try:
                out = subprocess.check_output(self.run_cmd, stdin=file, encoding='utf-8')
            except subprocess.CalledProcessError as e:
                print('Runner failed to run:')
                print(self)
                print(e)
                raise
            return out

    def compile(self):
        if self.compile_cmd is None:
            return
        return subprocess.check_output(self.compile_cmd).decode()

def py_runner(src_path: str):
    return Runner([sys.executable, src_path], None)

def cpp_runner(src_path: str, bin_name: str):
    if bin_name[0] != '/':
        bin_name = './' + bin_name
    return Runner(
            [bin_name],
            # '-Wl,-z,stack-size=268435456' for stack size maybe
            [CC, '-Wall', '-Wshadow', '-Wextra', '-O2', '-o', bin_name, src_path]
            )

def compile_all():
    for runner in _ALL_EXECUTABLES:
        runner.compile()




#!/usr/bin/env python3

# Example add problem.
# Constraints:
#   main: T <= 100, A <= 100, B <= 100
#   bonus: T <= 1e5, A <= 1e12, B <= 1e12

from calico_lib import Problem, cpp_runner, py_runner, TestFileBase, MulticaseTestFile, Subproblem, Runner
from collections.abc import Collection, Iterable
from typing import NamedTuple
import random
import os
from os import path

from calico_lib.multicase import TestCaseBase

problem_dir = os.path.dirname(__file__)

p = Problem(
        'gta6',
        problem_dir, # problem is in the same directory as the python source file
        test_sets=[
            Subproblem('main', rank=1),
        ])

class TestCase(NamedTuple):
    E: str
    D: int
    M: int
    Y: int

solution = py_runner(path.join(problem_dir, 'submissions/accepted/gta6.py'))
solution2 = cpp_runner(
        path.join(problem_dir, 'submissions/accepted/gta6.cpp'),
        path.join(problem_dir, 'gta6.bin'))
validator1 = py_runner(path.join(problem_dir, 'scripts/validator_main.py'))
validator2 = py_runner(path.join(problem_dir, 'scripts/validator.py'))

@p.pre_gen_fn
def pre_gen():
    random.seed('6')

class TestFile(TestFileBase):
    def __init__(self, cases: Iterable[TestCase]) -> None:
        self.cases = list(cases)
        super().__init__()

    # @override
    def write_test_in(self):
        """Write the input file of this test case using print_test"""
        p.print_test(len(self.cases))
        for case in self.cases:
            p.print_test(case.E)
            p.print_test("{:04d}".format(case.Y), "{:02d}".format(case.M), "{:02d}".format(case.D))

    # @override
    def validate_test_in(self, infile: str):
        """Verify the test using an external validator."""
        #if 'main' in self.subproblems:
        #    validator1.exec_file(infile)
        #validator2.exec_file(infile)
        for c in self.cases:
            assert c.D >= 1 and c.D <= 31
            assert c.M >= 1 and c.M <= 12
            assert c.Y >= 0 and c.Y <= 2200


    # @override
    def write_test_out(self, infile: str):
        p.print_test(solution.exec_file(infile))

# adds to all subproblems by default
p.add_sample_test(TestFile([
    TestCase("calico", 9, 11, 2025),
    TestCase("big ben's bday", 6, 9, 2026),
    TestCase("six eight", 27, 12, 2026),
    TestCase("gta6 eve", 18, 11, 2026),
    TestCase("contest start", 31, 12, 2200)
]))

# edge cases
p.add_hidden_test(TestFile([
    TestCase("gta 6 anniversary", 26, 5, 2027),
    TestCase("afjafi", 31, 5, 2025),
    TestCase("67", 6, 7, 2026),
    TestCase("beginning of time", 1, 1, 0),
    TestCase("12", 12, 12, 12)
]))

# cases = []
# for i in range(80):
#     cases.append(TestCase(i+1, 80-i))

# p.add_hidden_test(TestFile(cases), 'iota')
    
# cases = []
# for i in range(100):
#     cases.append(TestCase(i+1, 10000-i))

# p.add_hidden_test(TestFile(cases), 'iota', subproblems=['bonus'])

def random_string(length: int) -> str:
    string = ""
    for _ in range(length):
        string += chr(ord('a') + random.randint(0, 25))
    return string

# more ways to add test cases
@p.hidden_test_generator(test_count=4)
def pure_random() -> TestFile:
    test = TestFile([])
    for i in range(100):
        E = random_string(random.randint(1, 100))
        test.cases.append(TestCase(E, random.randint(1, 31), random.randint(1, 12), random.randint(0, 2200)))
    return test

# @p.hidden_test_generator(test_count=4, subproblems=['bonus'])
# def pure_random2():
#     cases = (TestCase(random.randint(70, int(1e12)), random.randint(70, int(1e12))) for _ in range(100))
#     return TestFile(cases)

def main():
    # increase stack size for running solutions using heaving recursion
    # import resource
    # resource.setrlimit(resource.RLIMIT_STACK, (268435456, 268435456))

    # TODO: set seed
    #solution2.compile()
    p.run_cli()

    # p.init_problem()
    # p.create_all_tests()
    # p.create_zip()

if __name__ == '__main__':
    main()
